//
//
//  Solar Guide Lamp  0.6.1  /  main.c
//      The 4th experimental edition <MPPT & PIR>
//      Copyright(C)2009 Toyohiko Togashi, tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//  Description:
//		Solar cell to battery charge and night marker lamp.
//		Charging method is current mode MPPT.
//  Device: MSP430 F2013
//  Pin assign:
//		p1: Vcc
//		p2: P1.0 A0+ <- Vsc
//		p3: P1.1 A4+ <- Vbat
//		p4: P1.2 A1+ <- Ibat+
//		p5: P1.3 A1- <- Ibat-
//		p6: P1.4 A2+ <- PIR+
//		p7: P1.5 A2- <- PIR-
//		p8: P1.6     -> LED-blue / R2=green
//		p9: P1.7     -> LED-red  / R2=red
//		p10: RST/NMI
//		p11: TEST
//		p12: P2.7    -> LED-green / R2=blue
//		p13: P2.6    -> PWM output
//		p14: Vss
//	SolarCell: Any ~10V
//	Battery: Any ~10V ~75mA
//
//  Update history:
//      2009.04.02 Climb search method
//      2009.04.12 Append binary search method
//      2009.04.20 Software PWM RGB-LED drive and slow flash
//      2009.04.22 Rewrite, day/night control
//      2009.04.30 LED gamma table
//      2009.05.04 CPU Power reduce(LPM1->3, SMCLK 8->1MHz, Timer off)
//                  Day:600->150uA Night:770->360uA Sleep:510->6.3uA
//      2009.05.05 Tempture -> LED color
//      2009.05.07 Append PIR function, refer TI AppNote SLAA283A
//      2009.05.16 Rewrite, day/night/midnight/hibernate control
//      2009.05.28 PCB revision "R2" support
//      2009.06.03 1st release
//

#include "msp430.h"

// PCB rev
//#define R1
#define R2

// MPPT method
//#define FULLSCAN                      // High <--> Low sweeping
//#define CLIMBSRCH                     // Mount climb method
#define BINSRCH                         // Binary search method

// LED Color mode
//#define COLOR_TEMP                    // Tempture -> color
#define COLOR_VBAT                      // Battery voltage -> color

//#define PWMPERIOD 19                  // 400KHz / SMCLK=8MHz
//#define PWMHILIM  18
//#define PWMPERIOD 40                  // 200KHz
//#define PWMHILIM  39
//#define PWMPERIOD 80                  // 100KHz
//#define PWMHILIM  78
//#define PWMPERIOD 160                 // 50KHz
//#define PWMHILIM  157
//#define PWMPERIOD 320                 // 250KHz
//#define PWMHILIM  310
#define PWMPERIOD 19                    // 50KHz / SMCLK=1MHz
#define PWMHILIM  18                    // duty high limit
#define PWMLOLIM  1                     // duty low limit 
#define PWMSTEP   1                     // duty step

#define LPFQ      1
#define SAMPWAIT  0

#ifdef R1
#define LED_PORT_RED   P1OUT
#define LED_PORT_GREEN P2OUT
#define LED_PORT_BLUE  P1OUT
#define LED_BIT_RED    BIT7
#define LED_BIT_GREEN  BIT7
#define LED_BIT_BLUE   BIT6
#endif
#ifdef R2
#define LED_PORT_RED   P1OUT
#define LED_PORT_GREEN P1OUT
#define LED_PORT_BLUE  P2OUT
#define LED_BIT_RED    BIT7
#define LED_BIT_GREEN  BIT6
#define LED_BIT_BLUE   BIT7
#endif

#define VSC_SUNSET         10           // [mV] day->night Vsc threshold / boosted
#define VSC_SUNRISE       400           // [mV] night->day Vsc threshold / non boost
#define VSC_DAYTIME       500           // [mV] Low batt wakeup level / non boost
#define VBAT_LOWLIM      3300           // [mV] 1.1V*3cell
#define VBAT_HIGHLIM     4800           // [mV] 1.6V*3cell
#define INTERVAL_HIBERNATE 40           // *250[msec]
#define INTERVAL_DAYTIME   12           // *250[msec]
#define INTERVAL_NIGHT      4           // *250[msec]
#define INTERVAL_MIDNIGHT  10           // *250[msec]
#define LIGHT_TIME        240           // *60[sec] (LIGHT_WAVE * LIGHT_RESOLUTION * LIGHT_PWM_CNT * LIGHT_PWM_PERIOD + INTERVAL_NIGHT)
//#define LIGHT_TIME          1         // *DEBUG*
#define LIGHT_WAVE         10           // 1 unit wave
#define LIGHT_RESOLUTION   32           // Bright level "dutytbl[]"
#define LIGHT_PWM_TIME      3           // Pulse count
#define LIGHT_PWM_PERIOD 1024           // 
#define PIR_THRESHOLD     300           // Threshold for motion
#define PIR_ONESHOTTIME    40           // *250[msec]
//#define PIR_ONESHOTTIME     4         // *DEBUG*
#define PIR_SCANTIME       10           // 1 unit

#define RC_NORM          0              // Normal return
#define RC_LOWBAT        1              //
#define RC_DAY           2              //

#define MAX(a,b) ((a > b) ? a : b)
#define ABS(a) ((a > 0) ? a : -a)

// Common area
const int dutytbl[] = {                 // PWM duty table (off time)
   1024, 512, 341, 256, 204, 170, 146, 124,
    108,  90,  76,  64,  55,  47,  40,  34,
     29,  25,  21,  18,  15,  13,  11,   9,
      8,   7,   6,   5,   4,   3,   2,   1};
#ifdef COLOR_VBAT
#define COLOR_VBAT_NUM 12
const           int vbat2code[]  = {4050,3990,3950,3930,3910,3900,3890,3880,3840,3770,3680      };
const unsigned char vbat2red[]   = {  31,  20,   0,   0,   0,  20,  31,  31,  31,  31,  31,   0 };
const unsigned char vbat2green[] = {  31,  31,  31,  31,  31,  31,  31,  20,   0,   0,   0,   0 };
const unsigned char vbat2blue[]  = {  31,  31,  31,  15,   0,   0,   0,   0,   0,  15,  31,  31 };
//                                  WHTE SKBL LTBL BLGR GREN LIME YELW ORNG  RED PINK VILT BLUE
#endif
#ifdef COLOR_TEMP
#define COLOR_TEMP_NUM 8
//                                    0~   4~   8~  12~  16~  20~  24~  28"C~
const unsigned char temp2red[]   = {  31,  31,  31,   0,  20,  31,   0,   0 }; 
const unsigned char temp2green[] = {   0,  15,  31,  31,  31,  31,  31,   0 };
const unsigned char temp2blue[]  = {   0,   0,   0,   0,   0,  31,  31,  31 }; 
//                                   RED ORNG YELW GREN LIME WHTE LTBL BLUE
#endif
static int bpir = 0;                    // PIR last value
static char pmode = 0;                  // Sleep LPMx mode

//
// Wait sec
//
void wait10(int i) {
  WDTCTL = WDT_ADLY_250;                // Preset 1s interval by ACLK
  __enable_interrupt();
  IE1 |= WDTIE;                         // Interval interrupt ON
  for (; i > 0; --i) {
    switch(pmode) {
    case 0:
      LPM0;
      break;
    case 1:
      LPM1;
      break;
    case 2:
      LPM2;
      break;
    case 3:
      LPM3;
      break;
    default:
      LPM4;
      break;
    }
  }
  IE1 &= ~WDTIE;                        // Interval interrupt OFF
}
#pragma vector=WDT_VECTOR
__interrupt void WDT_VECTOR_ISR(void) {
    switch(pmode) {
    case 0:
      LPM0_EXIT;
    case 1:
      LPM1_EXIT;
    case 2:
      LPM2_EXIT;
    case 3:
      LPM3_EXIT;
    default:
      LPM4_EXIT;
    }
}

//
// A/D channel input
//
int getadc(char opt, char n, int w) {
    char i;
    long s;

    SD16CTL |= SD16REFON;               // Internal 1.2V Ref ON
    SD16INCTL0 = opt;
    wait10(w);
    SD16CCTL0 |= SD16IE;                // Interrupt enable
    __enable_interrupt();
    s = 0;
    for (i = 0; i < n; i++) {
        SD16CCTL0 |= SD16SC;            // Set bit to start conversion
        while(SD16CCTL0 & SD16SC) {     // wait one shot mode
          LPM0;
        }
        s += SD16MEM0;                  // Get data
    }
    SD16CCTL0 &= ~SD16IE;               // Interrupt disable
    SD16CTL &= ~SD16REFON;              // OFF
    return(s / n);                      // Average
}
int wk;
#pragma vector=SD16_VECTOR
__interrupt void SD16_VECTOR_ISR(void) {
  wk = SD16MEM0;                        // Save data
  LPM0_EXIT;
}

static int adoffset = 0;
void zeroadj(){
      SD16CTL |= SD16REFON;             // Internal 1.2V Ref ON
      adoffset = getadc(SD16INTDLY_0 | SD16GAIN_1 | SD16INCH_7, LPFQ, SAMPWAIT);
      SD16CTL &= ~SD16REFON;            // OFF
};

int getvsc(){
  int a;
      
  // Unit convert   (Value-Offset) * (Vref(600mV) / Resolution(15bit)) * DivideRate(21) / Gain(1)
  SD16AE  &= ~SD16AE1;                  // P1.1=A0- or A4+
  a = getadc(SD16INTDLY_0 | SD16GAIN_1 | SD16INCH_0, LPFQ, SAMPWAIT);
  SD16AE  |=  SD16AE1;                  // P1.1=A0- or A4+
  return((long)(a - adoffset) * 600 * 21 / 1 / 32768);
};

int getvbat(){
  int a;
      
  // Unit convert   (Value-Offset) * (Vref(600mV) / Resolution(15bit)) * DivideRate(20.6) / Gain(1)
  SD16AE  &= ~SD16AE2;                  // P1.2=A1+ or A4-
  a = getadc(SD16INTDLY_0 | SD16GAIN_1 | SD16INCH_4, LPFQ, SAMPWAIT);
  SD16AE  |=  SD16AE2;                  // P1.2=A1+ or A4-
  return((long)(a - adoffset) * 600 * 206 / 10 / 32768);
};

int getibat(){
  int a;
      
  // Unit convert   (Value-Offset) * (Vref(600mV) / Resolution(15bit)) * DivideRate(1) / Gain(8)
  a = getadc(SD16INTDLY_0 | SD16GAIN_8 | SD16INCH_1, LPFQ, SAMPWAIT);
  return((long)(a - adoffset) * 6000 *  1 / 8 / 32768);
};

int getpir(){
  int a;
      
  a = getadc(SD16INTDLY_0 | SD16GAIN_4 | SD16INCH_2, LPFQ, SAMPWAIT);
  return(a);
};

int gettemp(){
  int a;
      
  a = getadc(SD16INTDLY_0 | SD16GAIN_4 | SD16INCH_2, LPFQ, SAMPWAIT);
  return((long)(a - adoffset - 19661) * 85 / 6280);
};

//
//   D a y t i m e
//
unsigned char day10() {
  unsigned char rc;
  int duty;                             // PWM duty rate
#ifndef BINSRCH
  int incr;                             // next duty offset
  int bibat;                            // before iBat
#endif
#ifdef BINSRCH
  int ibat1, ibat2, ibat3;              // before iBat
  int duty1, duty2, duty3;              // PWM duty rate
#endif
  
  // init
  rc     = RC_NORM;
  duty   = 1;
  pmode  = 1;
#ifndef BINSRCH
  incr   = PWMSTEP;
  bibat  = 0;
#endif

  // Timer -> PWM mode
  TACTL = 0                             // Timer control
//      | TASSEL_0                      // TACLK
//      | TASSEL_1                      // ACLK
        | TASSEL_2                      // SMCLK
//      | TASSEL_3                      // INCLK
        | ID_0                          // 1/1
//      | ID_1                          // 1/2
//      | ID_2                          // 1/4
//      | ID_3                          // 1/8
//      | MC_0                          // Stop
        | MC_1                          // Up mode
//      | MC_2                          // Continue mode
//      | MC_3                          // Up/down mode
//      | TACLR                         // All clear
//      | TAIE                          // Intrrupt enable
        ;
  TACCTL0 = CM_0;                       // Capture=off
  TACCTL1 = CM_0 | OUTMOD_7;            // Capture=off mode=reset/set
  TACCR0  = PWMPERIOD;                  // Period
  TACCR1  = duty;                       // Duty
  P2SEL  |= BIT6;                       // Set P2.6 to Timer

  // main loop
  for (;;) {
    int vsc;                            // [mV]
    int vbat;                           // [mV]
    int ibat;                           // [mA*10]

    // Get analog pin
    vsc  = getvsc();
    vbat = getvbat(); 
    ibat = getibat();

    // Vbat high limit check
    if (vbat < VBAT_LOWLIM) {           // Check low batt
      rc = RC_LOWBAT;
      break;

      // Vbat high limit check
    } else if (vbat > VBAT_HIGHLIM) {   // Check voltage
      duty = 0;                         // cut off

    // Night check
    } else if (vsc < VSC_SUNSET) {      // Check voltage night level
      break;                            // exit main loop

    // Day condition
    } else {
        if (duty == 0) {                // restart
          duty = 1;
        }
    }

    LED_PORT_GREEN |= LED_BIT_GREEN;

    // PWM duty update

#ifdef FULLSCAN
        duty += incr;
        if (duty < 0) {
          duty = 0;
          incr = -incr;
        } else if (duty >= PWMHILIM) {
          duty = PWMHILIM;
          incr = -incr;
        }
#endif
        
#ifdef CLIMBSRCH
        if (bibat > ibat) {
          incr = -incr;
        }
        duty += incr;
        bibat = ibat;
        if (duty < PWMLOLIM) {
          duty = PWMLOLIM;
          incr = -incr;
        } else if (duty >= PWMHILIM) {
          duty = PWMHILIM;
          incr = -incr;
        }
#endif
        
#ifdef BINSRCH
        if (duty > 0) {

          if (duty == duty1) {          // iBat set
            ibat1 = ibat;
          } else if (duty == duty2) {
            ibat2 = ibat;
          } else if (duty == duty3) {
            ibat3 = ibat;
          }
          
          if (duty == 1) {              // Reset
            ibat1 = -10;
            ibat2 = -10;
            ibat3 = -10;
            duty1 = PWMLOLIM;
            duty2 = PWMHILIM;
            duty3 = PWMHILIM * 2;
            duty = duty3;
          }

          if (ibat1 < ibat3) {          // right up
            if (ibat3 <= ibat2) {       // round
              if ((duty3 - duty1) > 2) {// check minimum range
                ibat1 = ibat2;
                duty1 = duty2;
                duty2 = (duty2 + duty3) / 2;
                duty  = duty2;          // next iBat point
              } else {                  // min range, duty1+1 == duty2 == duty3-1
                if (duty == duty1) {
                  duty = duty2;
                } else if (duty == duty2) {
                  duty = duty3;
                } else {
                  duty = duty1;
                }
              }
            } else {
              ibat1 = ibat2;
              duty1 = duty2;
              ibat2 = ibat3;
              duty2 = duty3;
              duty3 = duty2 + (duty2 - duty1);
              duty  = duty3;            // next iBat point
            }
          } else {                      // right down
            if (ibat1 <= ibat2) {       // round
              if ((duty3 - duty1) > 2) {// check minimum range
                ibat3 = ibat2;
                duty3 = duty2;
                duty2 = (duty1 + duty2) / 2;
                duty  = duty2;          // next iBat point
              } else {                  // min range, duty1+1 == duty2 == duty3-1
                if (duty == duty1) {
                  duty = duty2;
                } else if (duty == duty2) {
                  duty = duty3;
                } else {
                  duty = duty1;
                }
              }
            } else {
              ibat3 = ibat2;
              duty3 = duty2;
              ibat2 = ibat1;
              duty2 = duty1;
              duty1 = duty2 - (duty3 - duty2);
              duty  = duty1;            // next iBat point
            }
          }
          // Uncontrol condition check
          if (((duty1 + 1) == duty2) && (ABS(ibat1 - ibat2) > (ibat2 / 2))) {
            duty = 0;
//            LED_PORT_BLUE |= LED_BIT_BLUE;
          }
          if (((duty3 - 1) == duty2) && (ABS(ibat3 - ibat2) > (ibat2 / 2))) {
            duty = 0;
//            LED_PORT_RED |= LED_BIT_RED;
          }
          if ((duty1 == duty2)   || (duty2 == duty3) ||     // Fail safe
              (duty1 > PWMHILIM) || (duty3 < PWMLOLIM)) {
            duty = 0;
          }
        }
#endif

    // set PWM module
    if (duty != 0) {
      if (duty > PWMHILIM) {     // upper limit
        duty = PWMHILIM;
      }
      if (duty < PWMLOLIM) {     // lower limit
        duty = PWMLOLIM;
      }
    }
    TACCR1 = duty;
    
    LED_PORT_GREEN &= ~LED_BIT_GREEN;


    wait10(INTERVAL_DAYTIME);
  }

  // PWM off
  TACTL  = MC_0;                         // Timer stop
  P2OUT &= ~BIT6;                        // Set P2.6 to low
  P2SEL &= ~BIT6;                        // Set P2.6 to GIO
  
  return(rc);
}


//
//   N i g h t
//
unsigned char night10() {
  unsigned char rc;
  int tm;                               // Lighting time
  unsigned char cr, cb, cg;

  // init
  pmode = 3;                            // LPM3
  rc = RC_NORM;

  // Set color
#ifdef COLOR_TEMP
  {
    int temp;
    unsigned char t;

    temp = gettemp();
    t = (temp >> 2) & 7;              // 4"C step  0,4,8,12,16,20,24,28
    cr = temp2red[t];
    cg = temp2green[t];
    cb = temp2blue[t];
  }
#endif
#ifdef COLOR_VBAT
  {
    int vbat;
    unsigned char i;

    vbat = getvbat();
    for(i = 0; i < (COLOR_VBAT_NUM - 1); i++) {
      if (vbat > vbat2code[i]) break;
    }
    cr = vbat2red[i];
    cg = vbat2green[i];
    cb = vbat2blue[i];
  }
#endif
  
  for(tm = LIGHT_TIME; tm > 0; --tm) {
    int vsc;
    int vbat;

    // Get analog pin
    vsc  = getvsc();
    vbat = getvbat();

    // Battery check
    if (vbat < VBAT_LOWLIM) {
      rc = RC_LOWBAT;
      break;
    }

    // Morning check
    if (vsc > VSC_SUNRISE) {
      rc = RC_DAY;
      break;
    }
    
    // Lighting start
    {
      unsigned char h;

      
      // n wave
      for (h = 0; h < LIGHT_WAVE; h++) {
        unsigned char i;

        // One wave
        for (i = 0; i < (LIGHT_RESOLUTION*2); i++) {
            unsigned char j;
            int pr, pg, pb;
            unsigned char a;

            a  = (i < LIGHT_RESOLUTION) ? (i) : ((LIGHT_RESOLUTION*2) - i);
            pr = dutytbl[(int)cr * a / LIGHT_RESOLUTION];
            pg = dutytbl[(int)cg * a / LIGHT_RESOLUTION];
            pb = dutytbl[(int)cb * a / LIGHT_RESOLUTION];

            for (j = 0; j < LIGHT_PWM_TIME; j++) {
              int k;
              int r, g, b;

              r = 0;
              g = 0;
              b = 0;
              for (k = 0; k < LIGHT_PWM_PERIOD; k++) {
                if (--r < 0) {
                  LED_PORT_RED |=  LED_BIT_RED;
                  r = pr;
                } else {
                  LED_PORT_RED &= ~LED_BIT_RED;
                }

                if (--g < 0) {
                  LED_PORT_GREEN |=  LED_BIT_GREEN;
                  g = pg;
                } else {
                  LED_PORT_GREEN &= ~LED_BIT_GREEN;
                }

                if (--b < 0) {
                  LED_PORT_BLUE |=  LED_BIT_BLUE;
                  b = pb;
                } else {
                  LED_PORT_BLUE &= ~LED_BIT_BLUE;
                }
              }
            }
        }
      }
//      LED_PORT_RED   &= ~LED_BIT_RED;
//      LED_PORT_GREEN &= ~LED_BIT_GREEN;
//      LED_PORT_BLUE  &= ~LED_BIT_BLUE;
      
      // Pause
      wait10(INTERVAL_NIGHT);
    }
  }
  return(rc);
}

//
//   M i d n i g h t
//
unsigned char night20() {
  unsigned char rc;

  // init
  rc = RC_NORM;
  pmode = 3;
  
  for(;;) {
    int vsc;
    int vbat;
    unsigned char i;

    // Get analog pin
    vbat = getvbat();
    LED_PORT_BLUE |= LED_BIT_BLUE;
    vsc  = getvsc();
    LED_PORT_BLUE &= ~LED_BIT_BLUE;

    // Battery check
    if (vbat < VBAT_LOWLIM) {
      rc = RC_LOWBAT;
      break;
    }

    // Morning check
    if (vsc > VSC_SUNRISE) {
      rc = RC_DAY;
      break;
    }
    
    // PIR check
    for (i = 0; i < PIR_SCANTIME; i++) {
      int a;
      int b;
      
      a = getpir();
      b = a - bpir;
      if (b < 0) {
        b = -b;
      }
      bpir = a;
      if (b > PIR_THRESHOLD) {
        LED_PORT_RED   |= LED_BIT_RED;
        LED_PORT_GREEN |= LED_BIT_GREEN;
        LED_PORT_BLUE  |= LED_BIT_BLUE;
        wait10(PIR_ONESHOTTIME);
        LED_PORT_RED   &= ~LED_BIT_RED;
        LED_PORT_GREEN &= ~LED_BIT_GREEN;
        LED_PORT_BLUE  &= ~LED_BIT_BLUE;
      }
      wait10(INTERVAL_MIDNIGHT / PIR_SCANTIME);
    }
  }
  return(rc);
}

//
//   M a i n
//
int main(void) {
  //
  // Configure
  //

  // Watch dog or Interval timer
  WDTCTL  = WDTPW            // Watch dog pass code
          | WDTHOLD          // Stop watchdog timer
//        | WDTNMIES         // NMI edge rise
//        | WDTNMI           // NMI or RESET
//        | WDTTMSEL         // Interval timer mode
//        | WDTCNTCL         // Clear counter
//        | WDTSSEL          // Clock source ACLK or SMCLK
//                           // Divide 1/32768 1/8192 1/512 1/64
//        | WDTIS0           //           OFF    ON    OFF   ON
//        | WDTIS1           //           OFF    OFF   ON    ON
          ;
//__enable_interrupt();
//IE1 |= WDTIE;              // Interval interrupt ON

//DCOCTL  = CALDCO_8MHZ;     // DCO pre setting 1/8/12/16MHz
//BCSCTL1 = CALBC1_8MHZ;     // DCO pre setting 1/8/12/16MHz
  DCOCTL  = CALDCO_1MHZ;     // DCO pre setting 1/8/12/16MHz
  BCSCTL1 = CALBC1_1MHZ;     // DCO pre setting 1/8/12/16MHz
//DCOCTL  = 0                // DCO frequency shift
//                           //   -3 -2 -1  0 +1 +2 +3 +4
//        | DCO0             //    0  1  0  1  0  1  0  1
//        | DCO1             //    0  0  1  1  0  0  1  1
//        | DCO2             //    0  0  0  0  1  1  1  1
//                           // DCO fine tune
//                           //   0/32 1/32 2/32 ... 31/32
//        | MOD0             //     0   1    0         1
//        | MOD1             //     0   0    1         1
//        | MOD2             //     0   0    0         1
//        | MOD3             //     0   0    0         1
//        | MOD4             //     0   0    0         1
//            ;              //
//BCSCTL1 = 0                //
//        | XT2OFF           // XT2 off
//        | XTS              // LFXT1 High freq mode
//        | DIVA_0           // ACLK 1/1
//        | DIVA_1           // ACLK 1/2
//        | DIVA_2           // ACLK 1/4
//        | DIVA_3           // ACLK 1/8
//                           // DCO Frequency select
//        | RSEL0            //   0=120KHz 1=150KHz 2=210KHz 3=300KHz 4=410KHz
//        | RSEL1            //   5=580KHz 6=800KHz 7=1.15MHz 8=1.6MHz 9=2.3MHz
//        | RSEL2            //   10=3.4MHz 11=4.25MHz 12=5.8MHz 13=7.8MHz
//        | RSEL3            //   14=11.25MHz 15=15.25MHz
//        ;                  //
  BCSCTL2 = 0                // 
          | SELM_0           // MCKL source  DCO
//        | SELM_1           //              DCO
//        | SELM_2           //              XT2,LFXT1,VLO
//        | SELM_3           //              LFXT1
          | DIVM_0           // MCLK divide 1/1
//        | DIVM_1           //             1/2
//        | DIVM_2           //             1/4
//        | DIVM_3           //             1/8
//        | SELS             // SMCKL source = XT2,LFXT1,VLO (not=DCO)
          | DIVS_0           // SMCLK divide 1/1
//        | DIVS_1           //              1/2
//        | DIVS_2           //              1/4
//        | DIVS_3           //              1/8
//        | DCOR             // DCO register = external (not=internal)
          ;                  //
  BCSCTL3 = 0                //
//        | XT2S_0           // XT2 range 0.1~1MHz
//        | XT2S_1           //           1~3MHz
//        | XT2S_2           //           3~16MHz
//        | XT2S_3           //     external
//        | LFXT1S_0         // LFXT1/VLO 32768HzXtal (XTS:0.4~1MHz)
//        | LFXT1S_1         //           resv        (XTS:1~3MHz)
          | LFXT1S_2         //           VLO         (XTS:3~16MHz)
//        | LFXT1S_3         //           external
//        | XCAP_0           // LFXT1 Capaciter ~1pF
//        | XCAP_1           //                 ~6pF
//        | XCAP_2           //                 ~10pF
//        | XCAP_3           //                 ~12.5pF
          ;                  //

  // SD16 A/D
  SD16CTL = 0                           // Main control
//          | SD16REFON                 // Internal 1.2V Ref
            | SD16LP                    // Low power mode (20KHz~500KHz)
//          | SD16SSEL_0                // Clock select MCLK
//          | SD16SSEL_1                //              SMCLK
            | SD16SSEL_2                //              ACLK
//          | SD16SSEL_3                //              External
            | SD16DIV_0                 // Clock divide 1/1
//          | SD16DIV_1                 //              1/2
//          | SD16DIV_2                 //              1/4
//          | SD16DIV_3                 //              1/8
            | SD16XDIV_0                // Extend divide 1/1
//          | SD16XDIV_1                //               1/3
//          | SD16XDIV_2                //               1/16
//          | SD16XDIV_3                //               1/48
            ;
//SD16INCTL0 = 0
//           | SD16INTDLY_0             // Sample count 4
//           | SD16INTDLY_1             //              3
//           | SD16INTDLY_2             //              2
//           | SD16INTDLY_3             //              1
//           | SD16GAIN_1               // preamp x1
//           | SD16INCH_0               // input ch A0
//           | SD16INCH_1               //          A1
//           | SD16INCH_2               //          A2
//           | SD16INCH_3               //          A3
//           | SD16INCH_4               //          A4
//           | SD16INCH_5               //          Vdd
//           | SD16INCH_6               //          Temp
//           | SD16INCH_7               //          Zero
//           ;
  SD16CCTL0 = 0                         // A/D Control
//          | SD16UNI                   // unipolar mode (bipolar)
            | SD16DF                    // signed format
            | SD16SNGL                  // single shot mode (continueus)
//          | SD16SC                    // start conversion
//          | SD16IE                    // Intrrupt enable
            ;
  SD16AE = 0                            // Analog port select
         | SD16AE0                      // P1.0=A0+
         | SD16AE1                      // P1.1=A0- or A4+
         | SD16AE2                      // P1.2=A1+ or A4-
         | SD16AE3                      // P1.3=A1-
         | SD16AE4                      // P1.4=A2+
         | SD16AE5                      // P1.5=A2-
//       | SD16AE6                      // P1.6=A3+
//       | SD16AE7                      // P1.7=A3-
         ;

  // PWM off
  TACTL  = MC_0;                         // Timer stop

  // Port                               // 76543210 bit
  P1OUT = 0x00;                         // gr000000
  P2OUT = 0x00;                         // bt000000
  P1DIR = 0xca;                         // 11000000
  P2DIR = 0xc0;                         // 11000000
  P1SEL = 0x00;                         // 00000000
  P2SEL = 0x00;                         // 00000000
  P1REN = 0x00;                         // 00000000
  P2REN = 0x00;                         // 00000000


  // Year loop
  for (;;) {
    zeroadj();                          // A/D adjust

    // Wait for spring
    while(getvsc() < VSC_DAYTIME) {
      wait10(INTERVAL_HIBERNATE);
    }

    // Day loop
    for (;;) {

      // Daytime... Battery charge
      if (day10() == RC_LOWBAT) {
        break;
      }
      
      // Night... Slow flash
      if (night10() == RC_LOWBAT) {
        break;
      }
      
      // Midnight... PIR sense & light on
      if (night20() == RC_LOWBAT) {
        break;
      }

    }
  }
}
